package Browscap;

use strict;

use Data::Dumper;

sub new {
    my $class = shift;
    my $self = bless {}, $class;

    # load in the data
    my $BROWSCAPFILE = 'd:/james/browscap/browscap.ini';
    open (BROWSCAP, $BROWSCAPFILE) or die "Can't open Browscap data file";

    # Build up browser capabilities hash
    my (%browser, $browsername, $browsersubname, %browsersplit);
    while (<BROWSCAP>) {
	next unless ($_);  # skip empty lines 
	next if /^;/;      # skip comments

	if (/^\[(.*)\]$/) { 
	    my $browserstring = $1;
	    if (index($browserstring,'*') == -1) {
		# no wildcards in browsername
		$browsername = $1; # Store browser id for hash key
		$browsersubname = undef;
	    }
	    else {
		$browserstring =~ /^(.*)\*(.*)$/;
		$browsername = $1;
		$browsersubname = $2;
#		warn "$1 $2\n";
	    }	    
	}
	elsif ($browsername && /^([^=]+)=(.+)$/) {
	    # add capabilities under each browsername hash key
	    if (defined($browsersubname)) {
		# wildcards
		$browsersplit{$browsername}{$browsersubname}{$1} = $2;
	    } else { 
		$browser{$browsername}{$1} = $2; 
	    }
	}
    }
    close (BROWSCAP);

    $self->{browser} = \%browser;
    $self->{browsersplit} = \%browsersplit;

    $self;
}

sub property {
    my $self = shift;
    my $browsername = shift;
    my $propertyname = shift;

    (defined($browsername) && defined($propertyname)) or return undef;

    # exact match exists?
    no strict 'refs';
    if (exists($self->{browser}->{$browsername})) {
	if (exists($self->{browser}->{$browsername}->{$propertyname})) {
	    # return if exact match
	    return $self->{browser}->{$browsername}->{$propertyname};
	}
	elsif (exists($self->{browser}->{$self->{browser}->{$browsername}->{parent}}->{$propertyname})) {
	    # return if exact match in parent
	    return $self->{browser}->{$self->{browser}->{$browsername}->{parent}}->{$propertyname};
	}
    }
    use strict 'refs';

    # no exact match, do fuzzy matching

    foreach my $frontmatch (keys %{$self->{browsersplit}}) {
	if (index($browsername, $frontmatch) == 0) {  # front matches
	    my $restofmatch = substr($browsername, length($frontmatch));
	    foreach my $backmatch (keys %{$self->{browsersplit}->{$frontmatch}}) {
		if ($restofmatch =~ /^.*${backmatch}$/) { 
		if (exists($self->{browsersplit}->{$frontmatch}->{$backmatch}->{$propertyname})) {
		    return %{$self->{browsersplit}->{$frontmatch}->{$backmatch}->{$propertyname}};
		}
		elsif (exists($self->{browser}->{$self->{browsersplit}->{$frontmatch}->{$backmatch}->{parent}}->{$propertyname})) {
		    return $self->{browser}->{$self->{browsersplit}->{$frontmatch}->{$backmatch}->{parent}}->{$propertyname};
		}
	    }
	}
    }
}
    return undef;
}


1;

